﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 체크 라디오 버튼
        /// </summary>
        private RadioButton checkedRadioButton;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Loaded                         += Window_Loaded;
            this.imageRadioButton1.Checked += imageRadioButton_Checked;
            this.imageRadioButton2.Checked += imageRadioButton_Checked;
            this.imageRadioButton3.Checked += imageRadioButton_Checked;
            this.imageRadioButton4.Checked += imageRadioButton_Checked;
            this.applyButton.Click         += applyButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            #region Stretch 콤보 박스를 설정한다.

            string[] stretchArray = Enum.GetNames(typeof(Stretch));

            foreach(string stretch in stretchArray)
            {
                this.stretchComboBox.Items.Add(stretch);
            }

            this.stretchComboBox.SelectedItem = this.targetImageBrush.Stretch.ToString();

            #endregion
            #region AlignmentX 콤보 박스를 설정한다.

            string[] alignmentXArray = Enum.GetNames(typeof(AlignmentX));

            foreach(string alignmentX in alignmentXArray)
            {
                this.alignmentXComboBox.Items.Add(alignmentX);
            }

            this.alignmentXComboBox.SelectedItem = this.targetImageBrush.AlignmentX.ToString();

            #endregion
            #region AlignmentY 콤보 박스를 설정한다.

            string[] alignmentYArray = Enum.GetNames(typeof (AlignmentY));

            foreach(string alignmentY in alignmentYArray)
            {
                this.alignmentYComboBox.Items.Add(alignmentY);
            }

            this.alignmentYComboBox.SelectedItem = this.targetImageBrush.AlignmentY.ToString();

            #endregion
            #region TileMode 콤보 박스를 설정한다.

            string[] timeModeArray = Enum.GetNames(typeof(TileMode));

            foreach(string tileMode in timeModeArray)
            {
                this.tileModeComboBox.Items.Add(tileMode);
            }

            this.tileModeComboBox.SelectedItem = this.targetImageBrush.TileMode.ToString();

            #endregion
            #region ViewportUnits/ViewboxUnits 콤보 박스를 설정한다.

            string[] brushMappingModeArray = Enum.GetNames(typeof(BrushMappingMode));

            foreach(string brushMappingMode in brushMappingModeArray)
            {
                this.viewportUnitsComboBox.Items.Add(brushMappingMode);
                this.viewboxUnitsComboBox.Items.Add(brushMappingMode);
            }

            this.viewportUnitsComboBox.SelectedItem = this.targetImageBrush.ViewportUnits.ToString();
            this.viewboxUnitsComboBox.SelectedItem  = this.targetImageBrush.ViewboxUnits.ToString();

            #endregion

            this.viewportTextBox.Text = this.targetImageBrush.Viewport.ToString();
            this.viewboxTextBox.Text  = this.targetImageBrush.Viewbox.ToString();

            this.imageRadioButton1.IsChecked = true;
        }

        #endregion
        #region 이미지 라디오 버튼 체크시 처리하기 - imageRadioButton_Checked(sender, e)

        /// <summary>
        /// 이미지 라디오 버튼 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void imageRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            this.checkedRadioButton = sender as RadioButton;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region Apply 버튼 클릭시 처리하기 - applyButton_Click(sender, e)

        /// <summary>
        /// Apply 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void applyButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.targetImageBrush.ImageSource   = (this.checkedRadioButton.Content as Image).Source;
                this.targetImageBrush.Stretch       = (Stretch)Enum.Parse(typeof(Stretch), (string)this.stretchComboBox.SelectedItem);
                this.targetImageBrush.AlignmentX    = (AlignmentX)Enum.Parse(typeof(AlignmentX), (string)this.alignmentXComboBox.SelectedItem);
                this.targetImageBrush.AlignmentY    = (AlignmentY)Enum.Parse(typeof(AlignmentY), (string)this.alignmentYComboBox.SelectedItem);
                this.targetImageBrush.TileMode      = (TileMode)Enum.Parse(typeof(TileMode), (string)this.tileModeComboBox.SelectedItem);
                this.targetImageBrush.ViewportUnits = (BrushMappingMode)Enum.Parse(typeof(BrushMappingMode), (string)this.viewportUnitsComboBox.SelectedItem);
                this.targetImageBrush.ViewboxUnits  = (BrushMappingMode)Enum.Parse(typeof(BrushMappingMode), (string)this.viewboxUnitsComboBox.SelectedItem);

                RectConverter rectConverter = new RectConverter();

                string viewport = this.viewportTextBox.Text;

                if(!string.IsNullOrEmpty(viewport))
                {
                    this.targetImageBrush.Viewport = (Rect)rectConverter.ConvertFromString(viewport);
                }
                else
                {
                    this.targetImageBrush.Viewport = Rect.Empty;

                    this.viewportTextBox.Text = "Empty";
                }

                string viewbox = this.viewboxTextBox.Text;

                if(!string.IsNullOrEmpty(viewbox) && viewbox.ToLower() != "(auto)")
                {
                    this.targetImageBrush.Viewbox = (Rect)rectConverter.ConvertFromString(viewbox);
                }
                else
                {
                    this.viewboxTextBox.Text = "Empty";

                    this.targetImageBrush.Viewbox = Rect.Empty;
                }
            }
            catch(InvalidOperationException invalidOperationException)
            {
                MessageBox.Show("Invalid Viewport or Viewbox. " + invalidOperationException);
            }
            catch(FormatException formatException)
            {
                MessageBox.Show("Invalid Viewport or Viewbox. " + formatException);
            }
        }

        #endregion
    }
}